{% extends "base.html" %}

{% load i18n %}

{% block content %}
    {% if form.errors %}
    <p class="errorlist">{% trans "Sorry there are corrections needed in your form below:" %} {{ form.non_field_errors }}</p>
    {% endif %}
    {% if form.is_multipart %}
    <form enctype="multipart/form-data" method="post" action=".">
    {% else %}
    <form method="post" action=".">
   {% endif %}
   {{ form.as_p }}
   <input type="submit" value="Upload Zipped Shapefile" />
   </form>
{% endblock %}
